@@ -107,7 +107,7 @@ def is_websocket(headers):
     Determine whether a given set of headers is asking for WebSockets.
     """
 
-    return ("Upgrade" in headers.get("Connection", "")
+    return ("upgrade" in headers.get("Connection", "").lower()
             and headers.get("Upgrade").lower() == "websocket")
 
 def is_hybi00(headers):
@@ -224,6 +224,19 @@ def make_hybi07_frame(buf, opcode=0x1):
     frame = "%s%s%s" % (header, length, buf)
     return frame
 
+def make_hybi07_frame_dwim(buf):
+    """
+    Make a HyBi-07 frame with binary or text data according to the type of buf.
+    """
+    
+    # TODO: eliminate magic numbers.
+    if isinstance(buf, str):
+        return make_hybi07_frame(buf, opcode=0x2)
+    elif isinstance(buf, unicode):
+        return make_hybi07_frame(buf.encode("utf-8"), opcode=0x1)
+    else:
+        raise TypeError("In binary support mode, frame data must be either str or unicode")
+
 def parse_hybi07_frames(buf):
     """
     Parse HyBi-07 frames in a highly compliant manner.
@@ -327,11 +340,20 @@ class WebSocketProtocol(ProtocolWrapper):
     origin = "http://example.com"
     state = REQUEST
     flavor = None
+    do_binary_frames = False
 
     def __init__(self, *args, **kwargs):
         ProtocolWrapper.__init__(self, *args, **kwargs)
         self.pending_frames = []
 
+    def setBinaryMode(self, mode):
+        """
+        If True, send str as binary and unicode as text.
+        
+        Defaults to false for backwards compatibility.
+        """
+        self.do_binary_frames = bool(mode)
+
     def isSecure(self):
         """
         Borrowed technique for determining whether this connection is over
@@ -430,7 +452,10 @@ class WebSocketProtocol(ProtocolWrapper):
         if self.flavor == HYBI00:
             maker = make_hybi00_frame
         elif self.flavor in (HYBI07, HYBI10, RFC6455):
-            maker = make_hybi07_frame
+            if self.do_binary_frames:
+                maker = make_hybi07_frame_dwim
+            else:
+                maker = make_hybi07_frame
         else:
             raise WSException("Unknown flavor %r" % self.flavor)
 
@@ -460,18 +485,28 @@ class WebSocketProtocol(ProtocolWrapper):
             self.origin = self.headers["Origin"]
 
         # Check whether a codec is needed. WS calls this a "protocol" for
-        # reasons I cannot fathom.
-        protocol = None
+        # reasons I cannot fathom. Newer versions of noVNC (0.4+) sets
+        # multiple comma-separated codecs, handle this by chosing first one
+        # we can encode/decode.
+        protocols = None
         if "WebSocket-Protocol" in self.headers:
-            protocol = self.headers["WebSocket-Protocol"]
+            protocols = self.headers["WebSocket-Protocol"]
         elif "Sec-WebSocket-Protocol" in self.headers:
-            protocol = self.headers["Sec-WebSocket-Protocol"]
+            protocols = self.headers["Sec-WebSocket-Protocol"]
+
+        if isinstance(protocols, basestring):
+            protocols = [p.strip() for p in protocols.split(',')]
+
+            for protocol in protocols:
+                if protocol in encoders or protocol in decoders:
+                    log.msg("Using WS protocol %s!" % protocol)
+                    self.codec = protocol
+                    break
 
-        if protocol:
-            if protocol not in encoders or protocol not in decoders:
                 log.msg("Couldn't handle WS protocol %s!" % protocol)
+
+            if not self.codec:
                 return False
-            self.codec = protocol
 
         # Start the next phase of the handshake for HyBi-00.
         if is_hybi00(self.headers):
